==> flutter_idensic_mobile_sdk_plugin-1.13.1.tar.gz: android/src/main/kotlin/com/sumsub/idensic/mobile/sdk/plugin/MethodCallHandler.kt <==
package com.sumsub.idensic.mobile.sdk.plugin

import android.annotation.SuppressLint
import android.app.Activity
import android.util.Log
import java.util.Locale
import com.sumsub.sns.core.R
import com.sumsub.sns.core.SNSMobileSDK
import com.sumsub.sns.core.data.listener.TokenExpirationHandler
import com.sumsub.sns.core.data.model.SNSCompletionResult
import com.sumsub.sns.core.data.model.SNSException
import com.sumsub.sns.core.data.model.SNSSDKState
import com.sumsub.sns.core.data.model.SNSSupportItem
import com.sumsub.sns.liveness3d.SNSLiveness3d
import com.sumsub.sns.prooface.SNSProoface
import io.flutter.plugin.common.*
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.runBlocking

class MethodCallHandler(var activity: Activity?): MethodChannel.MethodCallHandler {

    private var methodChannel: MethodChannel? = null

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        if (call.method == "dismiss") {
            val snsSdk = SNSMobileSDK.Builder(activity!!, "", "")
                .build();

            snsSdk.dismiss();
        } else if (call.method == "onLaunchSDK") {
            val apiUrl = call.argument("apiUrl") ?: ""
            val flowName = call.argument("flowName") ?: ""
            val accessToken = call.argument("accessToken") ?: ""
            val languageCode = call.argument("languageCode") ?: ""
            val supportEmail = call.argument("supportEmail") ?: DEFAULT_EMAIL
            val isDebug = call.argument("isDebug") ?: false
            val modules = listOf(SNSLiveness3d(), SNSProoface())

            val onTokenExpirationHandler = object : TokenExpirationHandler {
                override fun onTokenExpired(): String? = runBlocking(Dispatchers.Main.immediate) {
                    val newToken = methodChannel?.invokeMethodBlock("onTokenExpiration") as? String
                    return@runBlocking newToken
                }
            }

            val onError = { exception: SNSException -> methodChannel?.invokeMethod("onError", exception.toMap())!! }
            val onStatusChanged = { newState: SNSSDKState, prevState: SNSSDKState -> methodChannel?.invokeMethod("onStatusChanged", listOf(newState.toStateString(), prevState.toStateString()))!! }
            val onCompleted = { r: SNSCompletionResult, state: SNSSDKState -> result.success(r.toMap(state)) }

            val supportItem = SNSSupportItem(
                    R.string.sns_support_EMAIL_title,
                    R.string.sns_support_EMAIL_description,
                    R.drawable.sns_ic_email,
                    SNSSupportItem.Type.Email,
                    supportEmail)

            val builder = SNSMobileSDK.Builder(activity!!, apiUrl, flowName)
                    .withAccessToken(accessToken, onTokenExpiration = onTokenExpirationHandler)
                    .withHandlers(onError = onError, onStateChanged = onStatusChanged, onCompleted = onCompleted)
                    .withSupportItems(listOf(supportItem))
                    .withModules(modules)
                    .withDebug(isDebug)
            
            if (languageCode.isNotEmpty()) {
                builder.withLocale(Locale(languageCode))
            }

            val snsSdk = builder.build();
            snsSdk.launch()
        } else {
            result.notImplemented()
        }
    }

    /**
     * Registers this instance as a method call handler on the given {@code messenger}.
     *
     * <p>Stops any previously started and unstopped calls.
     *
     * <p>This should be cleaned with {@link #stopListening} once the messenger is disposed of.
     */
    @SuppressLint("LogNotTimber")
    fun startListening(messenger: BinaryMessenger?) {
        if (methodChannel != null) {
            Log.wtf(TAG, "Setting a method call handler before the last was disposed.")
            stopListening()
        }
        methodChannel = MethodChannel(messenger, CHANNEL_NAME)
        methodChannel?.setMethodCallHandler(this)
    }

    /**
     * Clears this instance from listening to method calls.
     *
     * <p>Does nothing is {@link #startListening} hasn't been called, or if we're already stopped.
     */
    @SuppressLint("LogNotTimber")
    fun stopListening() {
        if (methodChannel == null) {
            Log.d(TAG, "Tried to stop listening when no methodChannel had been initialized.")
            return
        }

        methodChannel?.setMethodCallHandler(null)
        methodChannel = null
    }

    companion object {

        private const val TAG = "MethodCallHandler"
        private const val CHANNEL_NAME ="sumsub.com/flutter_idensic_mobile_sdk_plugin"
        private const val DEFAULT_EMAIL = "support@sumsub.com"
    }
}

fun SNSException.toMap(): Map<String, Any> {
    val type = when (this) {
        is SNSException.Api -> "Api"
        is SNSException.Network -> "Network"
        is SNSException.Unknown -> "Unknown"
    }

    val payload = when (this) {
        is SNSException.Api -> mapOf("code" to code, "description" to description, "correlationId" to correlationId)
        is SNSException.Network -> mapOf("message" to (wrap?.message ?: ""))
        is SNSException.Unknown -> mapOf("message" to (wrap?.message ?: ""))
    }

    return mapOf("type" to type, "payload" to payload)
}

fun SNSCompletionResult.toMap(state: SNSSDKState): Map<String, Any?> {
    val success = this is SNSCompletionResult.SuccessTermination
    var errorType: String? = null
    var errorMsg: String? = null
    if (state is SNSSDKState.Failed) {
        errorType = when (state) {
            is SNSSDKState.Failed.ApplicantNotFound -> "ApplicantNotFound"
            is SNSSDKState.Failed.ApplicantMisconfigured -> "ApplicantMisconfigured"
            is SNSSDKState.Failed.InitialLoadingFailed -> "InitialLoadingFailed"
            is SNSSDKState.Failed.InvalidParameters -> "InvalidParameters"
            is SNSSDKState.Failed.NetworkError -> "NetworkError"
            is SNSSDKState.Failed.Unauthorized -> "Unauthorized"
            else -> "Unknown"
        }

        errorMsg = state.message
        if (state.exception?.message != null) {
            errorMsg += ". ${state.exception?.message}"
        }
    }

    val actionResult = if (state is SNSSDKState.ActionCompleted) {
        mapOf("actionId" to state.actionId, "answer" to state.answer?.value)
    } else {
        null
    }

    return mapOf("success" to success, "status" to state.toStateString(), "errorType" to errorType, "errorMsg" to errorMsg, "actionResult" to actionResult)
}

fun SNSSDKState.toStateString(): String {
    return when (this) {
        is SNSSDKState.Approved -> "Approved"
        is SNSSDKState.Failed -> "Failed"
        is SNSSDKState.FinallyRejected -> "FinallyRejected"
        is SNSSDKState.Incomplete -> "Incomplete"
        is SNSSDKState.Initial -> "Initial"
        is SNSSDKState.Pending -> "Pending"
        is SNSSDKState.Ready -> "Ready"
        is SNSSDKState.TemporarilyDeclined -> "TemporarilyDeclined"
        is SNSSDKState.ActionCompleted -> "ActionCompleted"
        else -> "<unknown>"
    }
}

==> flutter_idensic_mobile_sdk_plugin-1.14.2.tar.gz: android/src/main/kotlin/com/sumsub/idensic/mobile/sdk/plugin/MethodCallHandler.kt <==
package com.sumsub.idensic.mobile.sdk.plugin

import android.annotation.SuppressLint
import android.app.Activity
import android.util.Log
import com.sumsub.sns.core.R
import com.sumsub.sns.core.SNSActionResult
import com.sumsub.sns.core.SNSMobileSDK
import com.sumsub.sns.core.data.listener.TokenExpirationHandler
import com.sumsub.sns.core.data.model.SNSCompletionResult
import com.sumsub.sns.core.data.model.SNSException
import com.sumsub.sns.core.data.model.SNSSDKState
import com.sumsub.sns.core.data.model.SNSSupportItem
import com.sumsub.sns.liveness3d.SNSLiveness3d
import com.sumsub.sns.prooface.SNSProoface
import io.flutter.plugin.common.*
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.runBlocking
import java.util.*

class MethodCallHandler(var activity: Activity?) : MethodChannel.MethodCallHandler {

    private var methodChannel: MethodChannel? = null

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        if (call.method == "dismiss") {
            val snsSdk = SNSMobileSDK.Builder(activity!!, "", "")
                    .build();

            snsSdk.dismiss();
        } else if (call.method == "onLaunchSDK") {
            val apiUrl = call.argument("apiUrl") ?: ""
            val flowName = call.argument("flowName") ?: ""
            val accessToken = call.argument("accessToken") ?: ""
            val languageCode = call.argument("languageCode") ?: ""
            val supportEmail = call.argument("supportEmail") ?: DEFAULT_EMAIL
            val isDebug = call.argument("isDebug") ?: false
            val modules = listOf(SNSLiveness3d(), SNSProoface())

            val onTokenExpirationHandler = object : TokenExpirationHandler {
                override fun onTokenExpired(): String? = runBlocking(Dispatchers.Main.immediate) {
                    return@runBlocking methodChannel?.invokeMethodBlock("onTokenExpiration") as? String
                }
            }

            val onError = { exception: SNSException -> methodChannel?.invokeMethod("onError", exception.toMap())!! }
            val onStatusChanged: ((state: SNSSDKState, prevState: SNSSDKState) -> Unit)? =
                if (call.argument("hasOnStatusChanged") as? Boolean == true) {
                    { newState: SNSSDKState, prevState: SNSSDKState ->
                        methodChannel?.invokeMethod("onStatusChanged", listOf(newState.toStateString(), prevState.toStateString()))!!
                    }
                } else {
                    null
                }
            val onCompleted = { r: SNSCompletionResult, state: SNSSDKState -> result.success(r.toMap(state)) }

            val onActionResult: ((String?, String?) -> SNSActionResult)? =
                if (call.argument("hasOnActionResult") as? Boolean == true) {
                    { actionId, answer ->
                        runBlocking(Dispatchers.Main.immediate) {
                            return@runBlocking try {
                                val arguments = listOf(mapOf(
                                    "actionId" to actionId,
                                    "answer" to answer,
                                    "actionType" to "",
                                    "allowContinuing" to false
                                ))
                                val reaction = methodChannel?.invokeMethodBlock("onActionResult", arguments) as? String
                                if (reaction == "cancel") SNSActionResult.Cancel else SNSActionResult.Continue
                            } catch (e: Exception) {
                                Log.d("FLU", Log.getStackTraceString(e))
                                SNSActionResult.Continue
                            }
                        }
                    }
                } else null

            val supportItem = SNSSupportItem(
                R.string.sns_support_EMAIL_title,
                R.string.sns_support_EMAIL_description,
                R.drawable.sns_ic_email,
                SNSSupportItem.Type.Email,
                supportEmail)

            val builder = SNSMobileSDK.Builder(activity!!, apiUrl, flowName)
                    .withAccessToken(accessToken, onTokenExpiration = onTokenExpirationHandler)
                    .withHandlers(onError = onError, onStateChanged = onStatusChanged, onCompleted = onCompleted, onActionResult = onActionResult)
                    .withSupportItems(listOf(supportItem))
                    .withModules(modules)
                    .withDebug(isDebug)

            if (languageCode.isNotEmpty()) {
                builder.withLocale(Locale(languageCode))
            }

            val snsSdk = builder.build();
            snsSdk.launch()
        } else {
            result.notImplemented()
        }
    }

    /**
     * Registers this instance as a method call handler on the given {@code messenger}.
     *
     * <p>Stops any previously started and unstopped calls.
     *
     * <p>This should be cleaned with {@link #stopListening} once the messenger is disposed of.
     */
    @SuppressLint("LogNotTimber")
    fun startListening(messenger: BinaryMessenger?) {
        if (methodChannel != null) {
            Log.wtf(TAG, "Setting a method call handler before the last was disposed.")
            stopListening()
        }
        methodChannel = MethodChannel(messenger, CHANNEL_NAME)
        methodChannel?.setMethodCallHandler(this)
    }

    /**
     * Clears this instance from listening to method calls.
     *
     * <p>Does nothing is {@link #startListening} hasn't been called, or if we're already stopped.
     */
    @SuppressLint("LogNotTimber")
    fun stopListening() {
        if (methodChannel == null) {
            Log.d(TAG, "Tried to stop listening when no methodChannel had been initialized.")
            return
        }

        methodChannel?.setMethodCallHandler(null)
        methodChannel = null
    }

    companion object {

        private const val TAG = "MethodCallHandler"
        private const val CHANNEL_NAME = "sumsub.com/flutter_idensic_mobile_sdk_plugin"
        private const val DEFAULT_EMAIL = "support@sumsub.com"
    }
}

fun SNSException.toMap(): Map<String, Any> {
    val type = when (this) {
        is SNSException.Api -> "Api"
        is SNSException.Network -> "Network"
        is SNSException.Unknown -> "Unknown"
    }

    val payload = when (this) {
        is SNSException.Api -> mapOf("code" to code, "description" to description, "correlationId" to correlationId)
        is SNSException.Network -> mapOf("message" to (wrap?.message ?: ""))
        is SNSException.Unknown -> mapOf("message" to (wrap?.message ?: ""))
    }

    return mapOf("type" to type, "payload" to payload)
}

fun SNSCompletionResult.toMap(state: SNSSDKState): Map<String, Any?> {
    val success = this is SNSCompletionResult.SuccessTermination
    var errorType: String? = null
    var errorMsg: String? = null
    if (state is SNSSDKState.Failed) {
        errorType = when (state) {
            is SNSSDKState.Failed.ApplicantNotFound -> "ApplicantNotFound"
            is SNSSDKState.Failed.ApplicantMisconfigured -> "ApplicantMisconfigured"
            is SNSSDKState.Failed.InitialLoadingFailed -> "InitialLoadingFailed"
            is SNSSDKState.Failed.InvalidParameters -> "InvalidParameters"
            is SNSSDKState.Failed.NetworkError -> "NetworkError"
            is SNSSDKState.Failed.Unauthorized -> "Unauthorized"
            else -> "Unknown"
        }

        errorMsg = state.message
        if (state.exception?.message != null) {
            errorMsg += ". ${state.exception?.message}"
        }
    }

    val actionResult = if (state is SNSSDKState.ActionCompleted) {
        mapOf("actionId" to state.actionId, "answer" to state.answer?.value)
    } else {
        null
    }

    return mapOf("success" to success, "status" to state.toStateString(), "errorType" to errorType, "errorMsg" to errorMsg, "actionResult" to actionResult)
}

fun SNSSDKState.toStateString(): String {
    return when (this) {
        is SNSSDKState.Approved -> "Approved"
        is SNSSDKState.Failed -> "Failed"
        is SNSSDKState.FinallyRejected -> "FinallyRejected"
        is SNSSDKState.Incomplete -> "Incomplete"
        is SNSSDKState.Initial -> "Initial"
        is SNSSDKState.Pending -> "Pending"
        is SNSSDKState.Ready -> "Ready"
        is SNSSDKState.TemporarilyDeclined -> "TemporarilyDeclined"
        is SNSSDKState.ActionCompleted -> "ActionCompleted"
        else -> "<unknown>"
    }
}
